/*
 * dbtool - a simple but powerfull commandline interface
 *      to the GNU gdbm system, usefull for shellscripts
 *      which needs a database for data storage.
 *
 * Copyright (c) 2001 Thomas Linden, <tom@daemon.de>
 * All Rights Reserved.      Released under the GPL.
 *
 * $Id: dbtool.cc,v 1.2 2002/02/11 14:52:51 scip Exp $
 */


#include "dbtool.h"
#include "config.h"
#include "engine.h"
#include "cipher.h"

Config config;
cipher rijn;

int main(int argc, char *argv[]) {
  config.args(argc, argv);
  Engine engine;
  string pkg = PACKAGE;

  if(config.parse() != 0) {
    cerr << config.usage << ".\n";;
    exit(1);
  }
  else {
    switch(config.command) {
    case 'i':
      /* insert */
      if(config.key == "") {
	cerr << pkg << ": key required\n";
	exit(1);
      }
      engine.insert();
      break;
    case 'd':
      /* dump */
      engine.dump();
      break;
    case 'I':
      engine.stdin();
      break;
    case 's':
      /* select */
      if(config.key == "") {
	cerr << pkg << ": key required\n";
	exit(1);
      }
      engine.select();
      break;
    case 'S':
      /* regexp select */
      if(config.key == "") {
	cerr << pkg << ": key (regexp) required\n";
	exit(1);
      }
      engine.regexp();
      break;
    case 'r':
      /* remove */
      if(config.key == "") {
	cerr << pkg << ": key required\n";
	exit(1);
      }
      engine.remove();
      break;
    case 'u':
      /* update */
      if(config.key == "") {
	cerr << pkg << ": key required\n";
	exit(1);
      }
      engine.update();
      break;
    case 'v':
      /* version */
      cerr << "This is " << pkg << " version " << VERSION << " by Thomas Linden.\n";
      exit(1);
    case 'h':
      /* usage/help */
      cerr << config.usage << ".\n";
      exit(1);
    }
  }
  exit(0);
}



string readpass() {
  char *pass;
  char *envpass;
  envpass = getenv(PW);
  if(envpass != NULL) {
    string pw = envpass;
    return pw;
  }
  else {
#ifdef HAVE_GETPASS
    pass = getpass("passphrase: ");
    string password = pass;
    free(pass);
#else
    cout << "passphrase: ";
    string password;
    cin >> password;
#endif
    return password;
  }
}


