/*
 * dbtool - a simple but powerfull commandline interface
 *      to the GNU gdbm system, usefull for shellscripts
 *      which needs a database for data storage.
 *
 * Copyright (c) 2001 Thomas Linden, <tom@daemon.de>
 * All Rights Reserved.      Released under the GPL.
 *
 * $Id: engine.h,v 1.1 2002/02/11 14:33:26 scip Exp $
 */


#ifndef ENGINE_H
#define ENGINE_H

/* the default initial blocksize for the gdbm database, see gdbm(3) */
#define BLOCKSIZE 1024

/* the default filemode for file creation, will be xored to the users umask */
#define FILEMODE  0644




#ifdef HAVE_BERKELEY

#ifndef NO_SYSTEM_INCLUDES
#include <sys/types.h>
#include <stdlib.h>
#include <string.h>
#ifndef _MSC_VER
#include <unistd.h>
#endif
#endif

#include <db_cxx.h>

#else

# include <gdbm.h>

#endif

#include "config.h"
#include "cipher.h"

extern Config config;
extern cipher rijn;

/*
 * The Engine class will do all gdbm related stuff, i.e.
 * insert, search, create, remove and so on.
 * methods defined in engine.cc
 */
class Engine {
 private:
  void init(); /* init() will be called from every member once */
  string encode(const string& data);
  string decode(const string& data);

#ifdef HAVE_BERKELEY
  Db *db;
#else
  GDBM_FILE db;
#endif

  string pkg;

 public:
  Engine() {};
  void insert();
  void update();
  void remove();
  void select();
  void  stdin();
  void   dump();
  void regexp();
};


#endif // ENGINE_H
